%% ex_TMsim_Active_LowPass_Filter.m
%
% Reproduces the TM and MC results of the example "Active Low-Pass Filter"
% (see Fig. 5 and 6) in [1] Sec. VI
%
% [1] R. Trinchero, P. Manfredi, T. Ding and I. S. Stievano, "Combined
% Parametric and Worst Case Circuit Analysis via Taylor Models," in IEEE
% Tran. on Circuits and Systems I: Regular Papers, vol. 63, no. 7,
% pp. 1067-1078, July 2016.
%
%
% This example script is part of the tool TMsim (TMsim.zip)
% Last revised: 31.10.2016. 
% Created by: Riccardo Trinchero, Paolo Manfredi, Igor S. Stievano.
%
% TMsim - Copyright (c) 2016 Riccardo Trinchero, Paolo Manfredi, Igor
% Stievano All rights reserved.
%  
% Embedding the program code in any commercial software is strictly
% prohibited. If the code is used in a scientific work, then reference
% should me made to the following publication.
%  
% [1] R. Trinchero, P. Manfredi, T. Ding, I. S. Stievano, "Combined
% Parametric and Worst Case Circuit Analysis via Taylor Models", IEEE
% Trans. Circuits and Systems - I: regular papers, Vol. 63, No. 7, pp.
% 1067-1078, Jul. 2016. PDFlink
%  
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
% IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
% THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
% PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

% cleaning
close all;
clear all;
clc

% Frequency data
Nf   = 20;

% high-frequency portion
freq = linspace(4000,10000,Nf);

% low-frequency portion
%freq = linspace(0,3000,Nf);

w = 2*pi*freq;

% MC runs
Nruns = 10000; 

% Circuit parameters
G1  = 1/5.4779e3;
G2  = 1/2.0076e3;
G3  = 1/3.3e3;   
G4  = G3; 
G5  = 1/4.5898e3;
G6  = 1/4.44e3;
G7  = 1/5.9999e3;
G8  = G3;
G9  = G3;
G10 = 1/4.2573e3;
G11 = 1/3.2201e3;
G12 = 1/5.88327e3;
G13 = G3;
G14 = G3;
G15 = 1/5.62599e3;
G16 = 1/3.63678e3;
G17 = 1/1.0301e3;
G18 = G3;
G19 = G3;
G20 = 1/5.808498e3;

C1n = 12 * 1e-9;
C2n = 10 * 1e-9;
C3n = 6.8 * 1e-9;
C4n = C2n;
C5n = 4.7 * 1e-9;
C6n = C2n;
C7n = 6.8 * 1e-9;
C8n = C2n;
C9n = C2n;

% Variation & MC Values
delta = 0.05;

R21n = 1.22*1e3;
R21d = R21n * 0;

C1d = C1n*0;
C3d = C3n*0;
C5d = C5n*0;
C7d = C7n*0;

C2d = C2n*delta;
C4d = C4n*delta;
C6d = C6n*delta;
C8d = C8n*delta;

C9d = C9n*delta;

%% =========================================================================
% Monte Carlo (uncorrelated variation, 3 independent parameters)
% =========================================================================
tic

x1 = 2*rand(Nruns,1)-1;
x2 = 2*rand(Nruns,1)-1;
x3 = 2*rand(Nruns,1)-1;

E = 1;

H = zeros(Nf,Nruns);

dim_MNA = 17;
MNA = zeros(dim_MNA);

progressbar(0);

for J = 1:Nf
    for K=1: Nruns        

        R21 = R21n + R21d * x2(K);
        G21 = 1/R21;

        C1 = C1n + C1d * x2(K);

        C2 = C2n + C2d * x1(K);

        C3 = C3n + C3d * x2(K);

        C4 = C4n + C4d * x1(K);

        C5 = C5n + C5d * x2(K);

        C6 = C6n + C6d * x2(K);

        C7 = C7n + C7d * x2(K);

        C8 = C8n + C8d * x2(K);

        C9 = C9n + C9d * x3(K);

        %Circuit excitation
        A = zeros(17,1);
        A(1)= E * G1;

        MNA(1,1) =  G1 + G2 + G6; 
        MNA(1,2) = -G2;
        MNA(1,5) = -G6; 

        MNA(2,1) = -G2;
        MNA(2,2) =  G2 + 1j*w(J)*C1;
        MNA(2,3) = -1j*w(J)*C1;

        MNA(3,2) =  G3 + G4;
        MNA(3,3) = -G3;
        MNA(3,4) = -G4;

        MNA(4,2) =  G5 + 1j*w(J)*C2;
        MNA(4,4) = -G5;

        MNA(5,1) = -G6;
        MNA(5,5) =  G6 + G7 + G11;
        MNA(5,6) = -G7;
        MNA(5,9) = -G11;

        MNA(6,5) = -G7;
        MNA(6,6) =  G7 + 1j*w(J)*C3;
        MNA(6,7) = -1j*w(J)*C3;

        MNA(7,6) =  G8 + G9;
        MNA(7,7) = -G8;
        MNA(7,8) = -G9;

        MNA(8,6) =  G10 + 1j*w(J)*C4;
        MNA(8,8) = -G10;

        MNA(9,5) = -G11;
        MNA(9,9) =  G11 + G12 + G16;
        MNA(9,10)= -G12;
        MNA(9,13)= -G16;

        MNA(10,9)  = -G12;
        MNA(10,10) =  G12 + 1j*w(J)*C5;
        MNA(10,11) = -1j*w(J)*C5;

        MNA(11,10) =  G13 + G14;
        MNA(11,11) = -G13;
        MNA(11,12) = -G14;

        MNA(12,10) =  G15 + 1j*w(J)*C6;
        MNA(12,12) = -G15;

        MNA(13,9)  = -G16;
        MNA(13,13) =  G16 + G17 + G21;
        MNA(13,14) = -G17;
        MNA(13,17) = -G21;

        MNA(14,13) = -G17;
        MNA(14,14) =  G17 + 1j*w(J)*C7;
        MNA(14,15) = -1j*w(J)*C7;

        MNA(15,14) =  G18 + G19;
        MNA(15,15) = -G18;
        MNA(15,16) = -G19;

        MNA(16,14) =  G20 + 1j*w(J)*C8;
        MNA(16,16) = -G20;

        MNA(17,13) = -G21;

        MNA(17,17) =  G21 + 1j*w(J)*C9;

        tmp = inv(MNA) * A; 

        H(J,K) = tmp(17);            
    end
    progressbar(J/Nf);
end
    
MC_time=toc

% magnitude
figure(1)
plot(freq,20*log10(abs(H)),'k-','color',[0.8,0.8,0.8]); hold on;
hP_tmp(1) = plot(freq,20*log10(abs(H(:,1))),'k-','color',[0.8,0.8,0.8]); hold on;

% real part of H
figure(2)
hPtmp = plot(freq,real(H),'k-','color',[0.8,0.8,0.8]); hold on;
hP_r(1) = hPtmp(1);

% imag part of H
figure(3)
hPtmp = plot(freq,imag(H),'k-','color',[0.8,0.8,0.8]); hold on;
hP_i(1) = hPtmp(1);

%% =========================================================================
% Taylor model (order 5)
% =========================================================================

N_var = 3; % Number of variables
N_order = 5; % Expansion order

% inizializing TMsim
initialize_TMsim(N_order,N_var);

% defining the TM variables
RT21 = TaylorModel(R21n,R21d,2);

GT21 = TM_inv_scalar(RT21);

CT1 = TaylorModel(C1n,C1d,2);
CT2 = TaylorModel(C2n,C2d,1);
CT3 = TaylorModel(C3n,C3d,2);
CT4 = TaylorModel(C4n,C4d,1);
CT5 = TaylorModel(C5n,C5d,2);
CT6 = TaylorModel(C6n,C6d,2);
CT7 = TaylorModel(C7n,C7d,2);
CT8 = TaylorModel(C8n,C8d,2);
CT9 = TaylorModel(C9n,C9d,3);

% Taylor model of 1
OneT  = TaylorModel(1,0,1);

% Taylor model of -1
OnemT = TaylorModel(-1,0,1);

% Taylor model of 0
ZeroT = TaylorModel(0,0,1);

% Empty MNA matrix
dim_MNA = 17;

for K=1:dim_MNA
    for I=1:dim_MNA
        MNA_TM(K,I) = ZeroT;
    end
end

%Circuit excitation
A = zeros(17,1);
A(1) = E * G1;

% Save minimax of magnitude
BMag_best = zeros(Nf,2);

progressbar(0);
tic

% MNA and TMs
MNA_TM(1,1) = ZeroT + G1 + G2 + G6; 
MNA_TM(1,2) = ZeroT - G2;
MNA_TM(1,5) = ZeroT - G6; 

MNA_TM(2,1) = ZeroT - G2;

MNA_TM(3,2) = ZeroT + G3 + G4;
MNA_TM(3,3) = ZeroT - G3;
MNA_TM(3,4) = ZeroT - G4;

MNA_TM(4,4) = ZeroT - G5;

MNA_TM(5,1) = ZeroT - G6;
MNA_TM(5,5) = ZeroT + G6 + G7 + G11;
MNA_TM(5,6) = ZeroT - G7;
MNA_TM(5,9) = ZeroT - G11;

MNA_TM(6,5) = ZeroT - G7;

MNA_TM(7,6) = ZeroT + G8 + G9;
MNA_TM(7,7) = ZeroT - G8;
MNA_TM(7,8) = ZeroT - G9;

MNA_TM(8,8) = ZeroT - G10;

MNA_TM(9,5) = ZeroT - G11;
MNA_TM(9,9) = ZeroT + G11 + G12 + G16;
MNA_TM(9,10)= ZeroT - G12;
MNA_TM(9,13)= ZeroT - G16;

MNA_TM(10,9)  = ZeroT - G12;

MNA_TM(11,10) = ZeroT + G13 + G14;
MNA_TM(11,11) = ZeroT - G13;
MNA_TM(11,12) = ZeroT - G14;

MNA_TM(12,12) = ZeroT - G15;

MNA_TM(13,9)  = ZeroT - G16;
MNA_TM(13,13) = ZeroT + G16 + G17 + GT21;
MNA_TM(13,14) = ZeroT - G17;
MNA_TM(13,17) = ZeroT - GT21;

MNA_TM(14,13) = ZeroT - G17;

MNA_TM(15,14) = ZeroT + G18 + G19;
MNA_TM(15,15) = ZeroT - G18;
MNA_TM(15,16) = ZeroT - G19;

MNA_TM(16,16) = ZeroT - G20;

MNA_TM(17,13) = ZeroT + OnemT * GT21;

for J = 1:Nf
       
    MNA_TM(2,2)   = ZeroT + G2 + 1j*w(J)*CT1;
    MNA_TM(2,3)   = ZeroT - 1j*w(J)*CT1;

    MNA_TM(4,2)   = ZeroT + G5 + 1j*w(J)*CT2;

    MNA_TM(6,6)   = ZeroT + G7 + 1j*w(J)*CT3;
    MNA_TM(6,7)   = ZeroT - 1j*w(J)*CT3;

    MNA_TM(8,6)   = ZeroT + G10 + 1j*w(J)*CT4;

    MNA_TM(10,10) = ZeroT + G12 + 1j*w(J)*CT5;
    MNA_TM(10,11) = ZeroT - 1j*w(J)*CT5;

    MNA_TM(12,10) = ZeroT + G15 + 1j*w(J)*CT6;

    MNA_TM(14,14) = ZeroT + G17 + 1j*w(J)*CT7;
    MNA_TM(14,15) = ZeroT - 1j*w(J)*CT7;

    MNA_TM(16,14) = ZeroT + G20 + 1j*w(J)*CT8;

    MNA_TM(17,17) = ZeroT + GT21 + 1j*w(J)*CT9;                                                       

    % inverting the MNA matrix
    invMNA_TM = Sherman_inv_matrix(MNA_TM);

    % solving the circuit
    tmp = invMNA_TM * A;

    HT = tmp(17);

    % max and min of both the real and imag part
    HT_r.min(J) = real(HT.B(1) + HT.I(1));
    HT_i.min(J) = imag(HT.B(1) + HT.I(1));
    HT_r.max(J) = real(HT.B(2) + HT.I(2));
    HT_i.max(J) = imag(HT.B(2) + HT.I(2));

    % max and min of the magnitude
    BMag_best(J,:) = TM_abs_bound(HT);


    progressbar(J/Nf);

end

TM_time=toc

% Magnitude
figure(1)
hP_tmp(2) = plot(freq,BMag_best(:,1),'m--','linewidth',1.5); hold on;
plot(freq,BMag_best(:,2),'m--','linewidth',1.5); hold on;
xlim([min(freq),max(freq)])

title('Magnitude','fontsize',16,'fontname','Times New Roman')
xlabel('$f$ [Hz]','interpreter','Latex','fontsize',16,'fontname','Times New Roman');
ylabel('Pass-band Ripple','fontsize',16,'fontname','Times New Roman');
set(gca,'fontname','Times New Roman','fontsize',16); 
legend(hP_tmp,'MC (indep.)','TM','location','southwest');

% Real Part
figure(2)
hP_r(2) = semilogx(freq,HT_r.min,'m--','linewidth',1.5); hold on;
semilogx(freq,HT_r.max,'m--','linewidth',1.5); hold on;
set(gca,'xlim',[freq(1),freq(end)]);
xlim([min(freq),max(freq)])

title('Real','fontsize',16,'fontname','Times New Roman')
xlabel('$f$ [Hz]','interpreter','Latex','fontsize',16,'fontname','Times New Roman');
ylabel('Pass-band Ripple','fontsize',16,'fontname','Times New Roman');
set(gca,'fontname','Times New Roman','fontsize',16); 
legend(hP_r,'MC (indep.)','TM','location','southwest');

% Imag Part
figure(3)
hP_i(2) = semilogx(freq,HT_i.min,'m--','linewidth',1.5); hold on;
semilogx(freq,HT_i.max,'m--','linewidth',1.5); hold on;
set(gca,'xlim',[freq(1),freq(end)]);
xlim([min(freq),max(freq)])

title('Imag','fontsize',16,'fontname','Times New Roman')
xlabel('$f$ [Hz]','interpreter','Latex','fontsize',16,'fontname','Times New Roman');
ylabel('Pass-band Ripple','fontsize',16,'fontname','Times New Roman');
set(gca,'fontname','Times New Roman','fontsize',16); 

legend(hP_i,'MC (indep.)','TM','location','southwest');
