%% ex_TMsim_nonlinear_op.m
%
% Reproduces the TM and MC results of the illustative example on the
% nonlinear operations (see Fig. 1) in [2] Sec. VI.E
%
% [2] R. Trinchero, P. Manfredi, I. S. Stievano "TMsim: an algorithmic tool
% for the parametric and worst-case simulation of systems with
% uncertainties," submitted to Hindawi, Mathematical Problems in
% Engineering.
% 
% This example script is part of the tool TMsim (TMsim.zip)
% Last revised: 31.10.2016. 
% Created by: Riccardo Trinchero, Paolo Manfredi, Igor S. Stievano.
%
% TMsim - Copyright (c) 2016 Riccardo Trinchero, Paolo Manfredi, Igor
% Stievano All rights reserved.
%  
% Embedding the program code in any commercial software is strictly
% prohibited. If the code is used in a scientific work, then reference
% should me made to the following publication.
%  
% [1] R. Trinchero, P. Manfredi, T. Ding, I. S. Stievano, "Combined
% Parametric and Worst Case Circuit Analysis via Taylor Models", IEEE
% Trans. Circuits and Systems - I: regular papers, Vol. 63, No. 7, pp.
% 1067-1078, Jul. 2016. PDFlink
%  
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
% IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
% THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
% PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

% cleaning
clear all
clc
close all

% defining variable domain
% x = [3,9];
x0 = 6;
dx = 3;

%% ========================================================================
% Monte Carlo 
% =========================================================================

Nruns = 201;
xdata = linspace(x0-dx,x0+dx,201);

% deterministic simulation ydata over the domain x_data=[3,9] = sin(1/(log(sqrt(xdta)))
ydata = sin(1./log(sqrt(xdata)));


%% =========================================================================
% Taylor model 
% =========================================================================

%defining expansion order
N_order = 6;

% inizializing TMsim
initialize_TMsim(N_order,1);

% defining the TM variables
x_TM = TaylorModel(x0,dx,1);

% applying nonlinear operations
y_TM = sin(inv(log(sqrt(x_TM))));

% normalized variable x_tilde (see Sec. IV of the paper)
x_tilde = (2*xdata-x0-dx-x0+dx)/(2*dx);

% evaluating the polinomial part
for ii=1:length(xdata)
    Pval(ii) = evalpoly(y_TM.P,x_tilde(ii));
end


figure
plot(xdata,ydata,'b'); hold on;
plot(xdata,Pval,'r--');
plot(xdata,Pval+y_TM.I(1),'g-');
plot(xdata,Pval+y_TM.I(2),'g-');
legend('g(x)','T_g.P(x)','TM bounds')

