% Last revised: 31.10.2016. 
% Created by: Riccardo Trinchero, Paolo Manfredi, Igor S. Stievano.
%
% TMsim - Copyright (c) 2016 Riccardo Trinchero, Paolo Manfredi, Igor
% Stievano All rights reserved.
%  
% Embedding the program code in any commercial software is strictly
% prohibited. If the code is used in a scientific work, then reference
% should me made to the following publication.
%  
% [1] R. Trinchero, P. Manfredi, T. Ding, I. S. Stievano, "Combined
% Parametric and Worst Case Circuit Analysis via Taylor Models", IEEE
% Trans. Circuits and Systems - I: regular papers, Vol. 63, No. 7, pp.
% 1067-1078, Jul. 2016. PDFlink
%  
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
% IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
% THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
% PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


classdef IntervalArithmetic
    properties
        inf
        sup
    end
    
    methods
        function out = IntervalArithmetic(a,b)
            if nargin>0
                if nargin==2
                    out.inf = a;
                    out.sup = b;
                elseif nargin==1
                    out.inf = a(1);
                    out.sup = a(2);
                end
            end
        end
        
        function out = plus(a,b)
            out = IntervalArithmetic;
            if ~isa(a,'IntervalArithmetic'), a = IntervalArithmetic(a,a); end
            if ~isa(b,'IntervalArithmetic'), b = IntervalArithmetic(b,b); end
            tmp = IA_sum([a.inf,a.sup],[b.inf,b.sup]);
            out.inf = tmp(1);
            out.sup = tmp(2);
        end
        
        function out = minus(a,b)
            out = IntervalArithmetic;
            if ~isa(a,'IntervalArithmetic'), a = IntervalArithmetic(a,a); end
            if ~isa(b,'IntervalArithmetic'), b = IntervalArithmetic(b,b); end
            tmp = IA_sum([a.inf,a.sup],IA_prod([b.inf,b.sup],[-1,-1]));
            out.inf = tmp(1);
            out.sup = tmp(2);
        end
        
        function out = mtimes(a,b)
            out = IntervalArithmetic;
            if ~isa(a,'IntervalArithmetic'), a = IntervalArithmetic(a,a); end
            if ~isa(b,'IntervalArithmetic'), b = IntervalArithmetic(b,b); end
            tmp = IA_prod([a.inf,a.sup],[b.inf,b.sup]);
            out.inf = tmp(1);
            out.sup = tmp(2);
        end
        
        function out = mrdivide(a,b)
            out = IntervalArithmetic;
            if ~isa(a,'IntervalArithmetic'), a = IntervalArithmetic(a,a); end
            if ~isa(b,'IntervalArithmetic'), b = IntervalArithmetic(b,b); end
            tmp = IA_div([a.inf,a.sup],[b.inf,b.sup]);
            out.inf = tmp(1);
            out.sup = tmp(2);
        end
        
        function out = mpower(a,k)
            out = IntervalArithmetic;
            tmp = IA_pow([a.inf,a.sup],k);
            out.inf = tmp(1);
            out.sup = tmp(2);
        end
        
   end
end