function BMag_best = TM_abs_bound(HT,scale)

% Tout = TM_abs(HT) calculates the bounds of abs of the Taylor model T1 with 
% complex coefficients

% Last revised: 31.10.2016. 
% Created by: Riccardo Trinchero, Paolo Manfredi, Igor S. Stievano.
%
% TMsim - Copyright (c) 2016 Riccardo Trinchero, Paolo Manfredi, Igor
% Stievano All rights reserved.
%  
% Embedding the program code in any commercial software is strictly
% prohibited. If the code is used in a scientific work, then reference
% should me made to the following publication.
%  
% [1] R. Trinchero, P. Manfredi, T. Ding, I. S. Stievano, "Combined
% Parametric and Worst Case Circuit Analysis via Taylor Models", IEEE
% Trans. Circuits and Systems - I: regular papers, Vol. 63, No. 7, pp.
% 1067-1078, Jul. 2016. PDFlink
%  
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
% IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
% THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
% PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


    % Computing the real and imag part of the Taylor Model HT
    HT_Re = TaylorModel;   % TM of real part of H
    HT_Re.P = real(HT.P);
    HT_Re.B = real(HT.B);
    HT_Re.I = real(HT.I);

    HT_Im = TaylorModel;  % TM of imaginary part of H
    HT_Im.P = imag(HT.P);
    HT_Im.B = imag(HT.B);
    HT_Im.I = imag(HT.I);
    

    %Calculation with IA
    HT_r.min = HT_Re.B(1) + HT_Re.I(1);
    HT_i.min = HT_Im.B(1) + HT_Im.I(1);
    HT_r.max = HT_Re.B(2) + HT_Re.I(2);
    HT_i.max = HT_Im.B(2) + HT_Im.I(2);

    Re2 = IA_pow([HT_r.min,HT_r.max],2); % square of real part
    Im2 = IA_pow([HT_i.min,HT_i.max],2); % square of imaginary part
    Hmag2 = Re2 + Im2;
    
    %Calculation with TM
    HTMag2 = TM_pow(HT_Re,2)+TM_pow(HT_Im,2);
    
    if nargin<2 || strcmpi(scale,'dB')            

        % Bound in dB (20*log10(.)) of magnitude via IA
        BMag_IA = 10*log10(Hmag2);       

        % Bound in dB (20*log10(.)) of magnitude via TM   
        %(fixed to avoid negative estimations)
        BMag_TM = 10*log10(max(HTMag2.B+HTMag2.I,1e-30)); 

    elseif strcmpi(scale,'lin')
                
        % Bound of the magnitude in linear scale sqrt(Re^2+Im^2) via IA       
        BMag_IA = sqrt(Hmag2);

        % Bound of the magnitude in linear scale sqrt(Re^2+Im^2) via TM       
        %BMag_TM = sqrt(max(HTMag2.B+HTMag2.I,0)); % fix to avoid negative estimations        
        
        Mag_TM = sqrt(HTMag2); % fix to avoid negative estimations
        
        %keyboard
                       
        if isnan(Mag_TM.B(1)+Mag_TM.I(1))
            BMag_TM(1) = 0;
        else
            BMag_TM(1) = Mag_TM.B(1)+Mag_TM.I(1);
        end
        
        if isnan(Mag_TM.B(2)+Mag_TM.I(2))
            BMag_TM(2) = 1e9;
        else
            BMag_TM(2) = Mag_TM.B(2)+Mag_TM.I(2);
        end
        
        BMag_TM = max(BMag_TM,0);
    else
        error('ABS specification not correct')
    end
    

    BMag_best(1) = max(BMag_IA(1),BMag_TM(1));
    
    BMag_best(2) = min(BMag_IA(2),BMag_TM(2));
end


