function [Tout] = TM_exp(T1)

% Tout = TM_exp(T1) calculates the Taylor model representation of the
% exponential of the Taylor model T1

% Last revised: 31.10.2016. 
% Created by: Riccardo Trinchero, Paolo Manfredi, Igor S. Stievano.
%
% TMsim - Copyright (c) 2016 Riccardo Trinchero, Paolo Manfredi, Igor
% Stievano All rights reserved.
%  
% Embedding the program code in any commercial software is strictly
% prohibited. If the code is used in a scientific work, then reference
% should me made to the following publication.
%  
% [1] R. Trinchero, P. Manfredi, T. Ding, I. S. Stievano, "Combined
% Parametric and Worst Case Circuit Analysis via Taylor Models", IEEE
% Trans. Circuits and Systems - I: regular papers, Vol. 63, No. 7, pp.
% 1067-1078, Jul. 2016. PDFlink
%  
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
% IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
% THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
% PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


global N_order;   % set order of the polynomial expansion

%real part
T1R = TaylorModel;

T1R.P = real(T1.P);
T1R.B = real(T1.B);
T1R.I = real(T1.I);

%imag part
T1I = TaylorModel;

T1I.P = imag(T1.P);
T1I.B = imag(T1.B);
T1I.I = imag(T1.I);

%Taylor expansion fot the real part
% center of the expansion
cf = T1R.P(1);
    
T1_bar = T1R-cf; 

% new TM variable Tout
ToutR = TaylorModel(1,0,1);
Tpwk= TaylorModel(1,0,1);

for k =1:N_order
    Tpwk =Tpwk*T1_bar;
    ToutR = ToutR+(1/factorial(k))*Tpwk;
end    

ToutR = ToutR*exp(cf);

%remiender

%lagrange
Tpwk =Tpwk*T1_bar;
tmp1 = exp(cf)*(1/factorial(N_order+1))*(Tpwk.B+Tpwk.I);
tmp2 = exp(T1_bar.B+T1_bar.I);
R1 = IA_prod(tmp1,tmp2);

ToutR.I = R1+ToutR.I;


%imaginary part of the exponent %exp(jT1I)=cos(T1I)+1j*sin(T1I);
ToutI = TM_cos(T1I)+1j*TM_sin(T1I);

% the output is exp(real(T1))*exp(j*imag(T1s))
Tout = ToutR*ToutI;







