function Tout = TM_inv_scalar(T1)

% Tout = inv_scalar(T1) calculates the Taylor model representation of the
% reciprocal of the Taylor model T1

% Last revised: 31.10.2016. 
% Created by: Riccardo Trinchero, Paolo Manfredi, Igor S. Stievano.
%
% TMsim - Copyright (c) 2016 Riccardo Trinchero, Paolo Manfredi, Igor
% Stievano All rights reserved.
%  
% Embedding the program code in any commercial software is strictly
% prohibited. If the code is used in a scientific work, then reference
% should me made to the following publication.
%  
% [1] R. Trinchero, P. Manfredi, T. Ding, I. S. Stievano, "Combined
% Parametric and Worst Case Circuit Analysis via Taylor Models", IEEE
% Trans. Circuits and Systems - I: regular papers, Vol. 63, No. 7, pp.
% 1067-1078, Jul. 2016. PDFlink
%  
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
% IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
% THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
% PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


global N_order;   % set order of the polynomial expansion

Tout = TaylorModel;

% center of the expansion
cf = T1.P(1);
    
T1_bar = TM_sum(T1,-cf);

% new TM variable Tout
Tout.P = zeros(size(T1.P));
Tout.P(1) = 1/cf;
Tout.B = [1/cf,1/cf];
Tout.I = [0,0];
    
Tout_tmp = TaylorModel(1,0,1);

T1_powk = 1;
% Compute the Taylor expansion for 1/T2 for order 
for k=1:N_order
    
    T1_powk = -1/cf*T1_powk*(T1_bar);
    
    Tout_tmp = T1_powk+Tout_tmp;        
end

Tout = Tout*Tout_tmp;


% Calculating the additional Remainder
%Lagrange
tmp1 = IA_inv(IA_pow([T1.B(1)+T1.I(1),T1.B(2)+T1.I(2)],N_order+2));
tmp2 = IA_pow(T1_bar.B+T1_bar.I,N_order+1);
R1 = IA_prod((-1)^(N_order+1)*tmp1,tmp2);

%Cauchy
T1_tilde = 1/cf*T1_bar;
tmp1 = IA_inv(T1_tilde.B+T1_tilde.I+[1,1]);
tmp2 = IA_pow(IA_prod([-1,-1],T1_tilde.B+T1_tilde.I),N_order+1);
R2 = IA_prod(1/cf*tmp1,tmp2);


% lower bound remainder
if abs((R1(1)))<abs((R2(1)))
    Tout.I(1) = ((R1(1))+Tout.I(1));
    %display('Lag')
else
    Tout.I(1) = ((R2(1))+Tout.I(1)); 
    %display('Cau')
end

% upper bound remainder
if abs((R1(2)))<abs((R2(2)))
    Tout.I(2) = (R1(2))+Tout.I(2);
    %display('Lag')
else
    Tout.I(2) = (R2(2))+Tout.I(2);    
    %display('Cau')
end
