function Tout = TM_log(T1)

% Tout = TM_log(T1) calculates the Taylor model representation
% of the logaritm of the Taylor model T1

% Last revised: 31.10.2016. 
% Created by: Riccardo Trinchero, Paolo Manfredi, Igor S. Stievano.
%
% TMsim - Copyright (c) 2016 Riccardo Trinchero, Paolo Manfredi, Igor
% Stievano All rights reserved.
%  
% Embedding the program code in any commercial software is strictly
% prohibited. If the code is used in a scientific work, then reference
% should me made to the following publication.
%  
% [1] R. Trinchero, P. Manfredi, T. Ding, I. S. Stievano, "Combined
% Parametric and Worst Case Circuit Analysis via Taylor Models", IEEE
% Trans. Circuits and Systems - I: regular papers, Vol. 63, No. 7, pp.
% 1067-1078, Jul. 2016. PDFlink
%  
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
% IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
% THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
% PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


global N_order; % set order of the polynomial expansion

cf = T1.P(1);

Tout = TaylorModel;

T1_bar = T1 - cf;

% constant term
Tout.P = zeros(size(T1.P));
Tout.P(1) = log(cf);
Tout.B = [log(cf),log(cf)];
Tout.I = [0,0];

% adding first order term
Tout = Tout +  T1_bar*(1/cf);

% adding additional terms
T1_powk = T1_bar;
for k = 2:(N_order)
    T1_powk = T1_powk*T1_bar;
    Tout = Tout + T1_powk*((-1)^(k+1)/k/cf^k);
end

% Calculating the additional Remainder
% Lagrange Remainder
tmp1 = IA_div(T1_bar.B+T1_bar.I,T1.B+T1.I);
R1 = IA_prod(((-1)^(N_order+2))/(N_order+1)*[1,1],IA_pow(tmp1,N_order+1));

%Chauchy Remainder
T1_tilde = 1/cf*T1_bar;

tmp1 = IA_div(IA_pow(T1_tilde.B+T1_tilde.I,N_order+1),1+T1_tilde.B+T1_tilde.I);

R2 = IA_prod((-1)^(N_order+2)*[1,1],tmp1);

% lower bound remainder
if (abs(real(R1(1)))<abs(real(R2(1))))
    Tout.I(1) = real(R1(1))+Tout.I(1);
    %display('Lagrange: lower')
else
    Tout.I(1) = real(R2(1))+Tout.I(1);
    %display('Chauchy: lower')

end

% upper bound remainder
if (abs(real(R1(2)))<abs(real(R2(2))))
    Tout.I(2) = real(R1(2))+Tout.I(2);
    %display('Lagrange: upper')

else
    Tout.I(2) = real(R2(2))+Tout.I(2);
    %display('Chauchy: upper')

end


