function Tout = TM_prod(T1,T2)

% Tout = TM_prod(T1,T2,x) calculates the Taylorm model representation of
% the product of Taylor models T1 and T2

% Last revised: 31.10.2016. 
% Created by: Riccardo Trinchero, Paolo Manfredi, Igor S. Stievano.
%
% TMsim - Copyright (c) 2016 Riccardo Trinchero, Paolo Manfredi, Igor
% Stievano All rights reserved.
%  
% Embedding the program code in any commercial software is strictly
% prohibited. If the code is used in a scientific work, then reference
% should me made to the following publication.
%  
% [1] R. Trinchero, P. Manfredi, T. Ding, I. S. Stievano, "Combined
% Parametric and Worst Case Circuit Analysis via Taylor Models", IEEE
% Trans. Circuits and Systems - I: regular papers, Vol. 63, No. 7, pp.
% 1067-1078, Jul. 2016. PDFlink
%  
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
% IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
% THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
% PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


global EXP XRANGE 
% Check if T1 or T2 is a scalar value

Tout = TaylorModel;

if ~isa(T1,'TaylorModel')
    
            if T1 == 0;
                Tout.P = zeros(size(T2.P));
                Tout.B = [0,0];
                Tout.I = [0,0];
                return
            end
                
            Tout.P = T1*T2.P;                       
            Tout.B = TM_bound_BN(Tout.P,EXP);
            Tout.I = IA_prod([T1,T1],T2.I);           
            
            return

else
        T1n = T1;
end

if ~isa(T2,'TaylorModel')   
    
            if T2 == 0;
                Tout.P = zeros(size(T1.P));
                Tout.B = [0,0];
                Tout.I = [0,0];
                return
            end
            
            Tout.P = T1.P*T2;   
            Tout.B = TM_bound_BN(Tout.P,EXP);
            Tout.I = IA_prod(T1.I,[T2,T2]); 
            
            return
else
        T2n = T2;    
end


    if or(myiszero(T1),myiszero(T2))
        Tout.P = zeros(size(T1.P));
        Tout.B = [0,0];
        Tout.I = [0,0];
        return
    end

    if iscalarTM(T1)
       Tout.P = T2.P*T1.P(1);              
       Tout.B = TM_bound_BN(Tout.P,EXP);
       Tout.I = IA_sum(IA_prod(T2.I,T1.B),IA_prod(T2.B,T1.I));
       return
    end
 
    if iscalarTM(T2)
       Tout.P = T1.P*T2.P(1);
       Tout.B = TM_bound_BN(Tout.P,EXP);
       Tout.I = IA_sum(IA_prod(T2.B,T1.I),IA_prod(T2.I,T1.B));
       return
    end

       
    % TM_bound
    [Tout.P,Be] = polyprod(T1n.P,T2n.P);

    % bound of the new polynomial
    Tout.B = TM_bound_BN(Tout.P,EXP);
        
    
    R1 = IA_prod(T1n.B,T2n.I);
    R2 = IA_prod(T2n.B,T1n.I);
    R3 = IA_prod(T1n.I,T2n.I);

    tmp = IA_sum(IA_sum(IA_sum(R1,R2),R3),Be);
     
    Tout.I = tmp;
    