function Tout_m = TM_prod_matrix(T1_m,T2_m)

% Tout_m = TM_prod_matrix(T1_m,T2_m) calculates the product of the Taylor 
% model matrices T1_m and T2_m.

% Last revised: 31.10.2016. 
% Created by: Riccardo Trinchero, Paolo Manfredi, Igor S. Stievano.
%
% TMsim - Copyright (c) 2016 Riccardo Trinchero, Paolo Manfredi, Igor
% Stievano All rights reserved.
%  
% Embedding the program code in any commercial software is strictly
% prohibited. If the code is used in a scientific work, then reference
% should me made to the following publication.
%  
% [1] R. Trinchero, P. Manfredi, T. Ding, I. S. Stievano, "Combined
% Parametric and Worst Case Circuit Analysis via Taylor Models", IEEE
% Trans. Circuits and Systems - I: regular papers, Vol. 63, No. 7, pp.
% 1067-1078, Jul. 2016. PDFlink
%  
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
% IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
% THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
% PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


[M1,N1] = size(T1_m);
[M2,N2] = size(T2_m);

Tout_m = TaylorModel;

% product scalar TM x matrix TM
if M1==1 && N1==1
    for ii=1:M2
        for jj=1:N2
            Tout_m(ii,jj) = TM_prod(T1_m,T2_m(ii,jj));
        end
    end

% product matrix TM x scalar TM
elseif M2==1 && N2==1
    for ii=1:M1
        for jj=1:N1
            Tout_m(ii,jj) = TM_prod(T1_m(ii,jj),T2_m);
        end
    end
else
    

if N1~=M2
    error('inner matrix dimensions must agree');
end



if isa(T1_m(1,1),'TaylorModel')
    ZeroT = TaylorModel;
    ZeroT.P=zeros(size(T1_m(1,1).P));
else
    ZeroT = TaylorModel;
    ZeroT.P=zeros(size(T2_m(1,1).P));
end

% Taylor model of 0
% ZeroT.P = zeros(2,2,2);
ZeroT.B = [0,0];
ZeroT.I = [0,0];

% Matrix product
for ii = 1:M1
    for jj = 1:N2
        
        Tout_m(ii,jj) = ZeroT;
        
        for kk = 1:N1
            
            if or(myiszero(T1_m(ii,kk)),myiszero(T2_m(kk,jj)))
                % do nothing
                % tmp = 0;
            else                
                
                tmp = TM_prod(T1_m(ii,kk),T2_m(kk,jj));
            
                if kk == 1
                    Tout_m(ii,jj) = tmp;
                else
                     Tout_m(ii,jj) = TM_sum(Tout_m(ii,jj),tmp);

                end
            end

        end
    end
end

end

return