function Tout = TM_sum(T1,T2)

% Tout = TM_sum(T1,T2) calculates the Taylor model represenatation of the 
% sum of Taylor models T1 and T2

% Last revised: 31.10.2016. 
% Created by: Riccardo Trinchero, Paolo Manfredi, Igor S. Stievano.
%
% TMsim - Copyright (c) 2016 Riccardo Trinchero, Paolo Manfredi, Igor
% Stievano All rights reserved.
%  
% Embedding the program code in any commercial software is strictly
% prohibited. If the code is used in a scientific work, then reference
% should me made to the following publication.
%  
% [1] R. Trinchero, P. Manfredi, T. Ding, I. S. Stievano, "Combined
% Parametric and Worst Case Circuit Analysis via Taylor Models", IEEE
% Trans. Circuits and Systems - I: regular papers, Vol. 63, No. 7, pp.
% 1067-1078, Jul. 2016. PDFlink
%  
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
% IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
% THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
% PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


global EXP;

Tout = TaylorModel;

% Check if T1 or T2 is a scalar value

if ~isa(T1,'TaylorModel')
    
        Tout.P = T2.P;
        
        Tout.P(1) = Tout.P(1)+T1;                     
        
        Tout.B = T2.B+T1;
        Tout.I = T2.I;

        return
    
else
        T1n = T1;
end

if ~isa(T2,'TaylorModel')
    
        Tout.P = T1.P;
        Tout.P(1) = Tout.P(1)+T2;        
        
        Tout.B = T1.B+T2;
        Tout.I = T1.I;
            
        return
    
else
        T2n = T2;
end


if iscalarTM(T1)
       Tout.P = T2.P;
       Tout.P(1) = T1.P(1)+T2.P(1); 
       Tout.B = T1.B+T2.B;
       Tout.I = T1.I+T2.I; 
       return
end
 
if iscalarTM(T2)
       Tout.P = T1.P;
       Tout.P(1) = T1.P(1)+T2.P(1); 
       Tout.B = T1.B+T2.B;
       Tout.I = T1.I+T2.I; 
       return
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% output polynomial is just the sum of polynomial coefficients
Tout.P = T1n.P + T2n.P; 

% calculating the bound of new polynomial
Tout.B = TM_bound_BN(Tout.P,EXP);
    
% the output IA-reminder is just the sum of the IA-reminders
Tout.I = T1n.I + T2n.I; 
