function B = TM_bound_BN(Pin,EXP)

% B = TM_bound_BN(Pin,EXP) computes the upper and lower bounds of a
% polynomial defined by the coefficients collected in the vector Pin and
% corresponding to combination of the variables defined by the matrix
% EXP

% Last revised: 31.10.2016. 
% Created by: Riccardo Trinchero, Paolo Manfredi, Igor S. Stievano.
%
% TMsim - Copyright (c) 2016 Riccardo Trinchero, Paolo Manfredi, Igor
% Stievano All rights reserved.
%  
% Embedding the program code in any commercial software is strictly
% prohibited. If the code is used in a scientific work, then reference
% should me made to the following publication.
%  
% [1] R. Trinchero, P. Manfredi, T. Ding, I. S. Stievano, "Combined
% Parametric and Worst Case Circuit Analysis via Taylor Models", IEEE
% Trans. Circuits and Systems - I: regular papers, Vol. 63, No. 7, pp.
% 1067-1078, Jul. 2016. PDFlink
%  
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
% IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
% THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
% PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


global N_order

N_var = size(EXP,1);

global BN_Base;

N_mon = size(EXP,2);

b = zeros(1,(N_order+1)^N_var);


for mon = 1:N_mon

    if Pin(mon) ~= 0
        for ii = 1:N_var
           KK(ii) = EXP(ii,mon) + 1; 
        end
        
        U   = 1;
        tmp = zeros(1,(N_order+1)^N_var);
% *************************************************************************            
        if N_var == 1
           for jj1 = 1:N_order+1
            
                 tmp(U) = tmp(U) + BN_Base(KK(1),jj1);
     
                 U = U + 1;
           end  
            
        end   
% *************************************************************************            
        if N_var == 2
           for jj1 = 1:N_order+1
              for jj2 = 1:N_order+1
                        
                 tmp(U) = tmp(U) + BN_Base(KK(1),jj1) * BN_Base(KK(2),jj2);
     
                 U = U + 1;
              end
           end  
            
        end
% *************************************************************************      
        if N_var == 3
            
           for jj1 = 1:N_order+1
              for jj2 = 1:N_order+1
                 for jj3 = 1:N_order+1
        
                 tmp(U) = tmp(U) + BN_Base(KK(1),jj1) * BN_Base(KK(2),jj2) * BN_Base(KK(3),jj3);
     
                 U = U + 1;
                 end
              end
           end 
        end
% *************************************************************************      
        if N_var == 4
            
           for jj1 = 1:N_order+1
              for jj2 = 1:N_order+1
                 for jj3 = 1:N_order+1
                    for jj4 = 1:N_order+1
               
                      tmp(U) = tmp(U) + BN_Base(KK(1),jj1) * BN_Base(KK(2),jj2) * BN_Base(KK(3),jj3) * BN_Base(KK(4),jj4);
                      
                      U = U + 1;
                    end
                 end
              end
           end 
        end        
% *************************************************************************      
        if N_var == 5
            
           for jj1 = 1:N_order+1
              for jj2 = 1:N_order+1
                 for jj3 = 1:N_order+1
                    for jj4 = 1:N_order+1
                       for jj5 = 1:N_order+1
               
                      tmp(U) = tmp(U) + BN_Base(KK(1),jj1) * BN_Base(KK(2),jj2) * BN_Base(KK(3),jj3)...
                          * BN_Base(KK(4),jj4,4) * BN_Base(KK(5),jj5,5);
                      
                      U = U + 1;
                       end
                    end
                 end
              end
           end 
        end                
% *************************************************************************      
        if N_var == 6
            
           for jj1 = 1:N_order+1
              for jj2 = 1:N_order+1
                 for jj3 = 1:N_order+1
                    for jj4 = 1:N_order+1
                       for jj5 = 1:N_order+1
                          for jj6 = 1:N_order+1
              
                      tmp(U) = tmp(U) + BN_Base(KK(1),jj1) * BN_Base(KK(2),jj2) * BN_Base(KK(3),jj3)...
                          * BN_Base(KK(4),jj4) * BN_Base(KK(5),jj5) * BN_Base(KK(6),jj6);
                      
                      U = U + 1;
                          end
                       end
                    end
                 end
              end
           end 
        end
% *************************************************************************      
        if N_var == 7
            
           for jj1 = 1:N_order+1
              for jj2 = 1:N_order+1
                 for jj3 = 1:N_order+1
                    for jj4 = 1:N_order+1
                       for jj5 = 1:N_order +1
                          for jj6 = 1:N_order+1
                             for jj7 = 1:N_order+1
               
                      tmp(U) = tmp(U) + BN_Base(KK(1),jj1) * BN_Base(KK(2),jj2) * BN_Base(KK(3),jj3)...
                            * BN_Base(KK(4),jj4) * BN_Base(KK(5),jj5) * BN_Base(KK(6),jj6) * BN_Base(KK(7),jj7);
                      
                      U = U + 1;
                             end
                          end
                       end
                    end
                 end
              end
           end 
        end        
% *************************************************************************      
        if N_var == 8
            
           for jj1 = 1:N_order+1
              for jj2 = 1:N_order+1
                 for jj3 = 1:N_order+1
                    for jj4 = 1:N_order+1
                       for jj5 = 1:N_order +1
                          for jj6 = 1:N_order+1
                             for jj7 = 1:N_order+1
                                for jj8 = 1:N_order+1
               
                      tmp(U) = tmp(U) + BN_Base(KK(1),jj1) * BN_Base(KK(2),jj2) * BN_Base(KK(3),jj3)...
                            * BN_Base(KK(4),jj4) * BN_Base(KK(5),jj5) * BN_Base(KK(6),jj6) * BN_Base(KK(7),jj7)...
                            * BN_Base(KK(8),jj8);
                      
                      U = U + 1;
                                end
                             end
                          end
                       end
                    end
                 end
              end
           end 
        end               
% *************************************************************************      
        if N_var == 9
            
           for jj1 = 1:N_order+1
              for jj2 = 1:N_order+1
                 for jj3 = 1:N_order+1
                    for jj4 = 1:N_order+1
                       for jj5 = 1:N_order +1
                          for jj6 = 1:N_order+1
                             for jj7 = 1:N_order+1
                                for jj8 = 1:N_order+1
                                   for jj9 = 1:N_order+1
               
                      tmp(U) = tmp(U) + BN_Base(KK(1),jj1) * BN_Base(KK(2),jj2) * BN_Base(KK(3),jj3)...
                            * BN_Base(KK(4),jj4) * BN_Base(KK(5),jj5) * BN_Base(KK(6),jj6) * BN_Base(KK(7),jj7)...
                            * BN_Base(KK(8),jj8) * BN_Base(KK(9),jj9);
                      
                      U = U + 1;
                                   end
                                end
                             end
                          end
                       end
                    end
                 end
              end
           end 
        end               
        
        b = b + Pin(mon) * tmp;

    end
end

B(1) = min(real(b)) + 1j*min(imag(b));
B(2) = max(real(b)) + 1j*max(imag(b));
  