function EXP = matrix_EXP(N_var,N_order)

% EXP = matrix_EXP(N_var,N_order) generates a matrix in which the entries 
% represent all the combination of the exponents of a multivarite polinomial 
% with N_var variables up to the order N_order

% Last revised: 31.10.2016. 
% Created by: Riccardo Trinchero, Paolo Manfredi, Igor S. Stievano.
%
% TMsim - Copyright (c) 2016 Riccardo Trinchero, Paolo Manfredi, Igor
% Stievano All rights reserved.
%  
% Embedding the program code in any commercial software is strictly
% prohibited. If the code is used in a scientific work, then reference
% should me made to the following publication.
%  
% [1] R. Trinchero, P. Manfredi, T. Ding, I. S. Stievano, "Combined
% Parametric and Worst Case Circuit Analysis via Taylor Models", IEEE
% Trans. Circuits and Systems - I: regular papers, Vol. 63, No. 7, pp.
% 1067-1078, Jul. 2016. PDFlink
%  
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
% IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
% THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
% PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


% Checking range
if N_var<1 || N_var>25
    error('number of variables out of range 1 - 25');
end

EXP = [];

if N_var == 1
    EXP = [0:N_order];
    
elseif N_var == 2
    for p1 = 0:N_order
        for p2 = 0:p1
            EXP(:,end+1) = [(p1-p2) p2]';
        end
    end
    
elseif N_var == 3
    for p1 = 0:N_order
        for p2 = 0:p1
            for p3 = 0:p2
                EXP(:,end+1) = [(p1-p2) (p2-p3) p3]';
            end
        end
    end
    
elseif N_var == 4
    for p1 = 0:N_order
        for p2 = 0:p1
            for p3 = 0:p2
                for p4 = 0:p3
                    EXP(:,end+1) = [(p1-p2) (p2-p3) (p3-p4) p4]';
                end
            end
        end
    end
    
elseif N_var == 5
    for p1 = 0:N_order
        for p2 = 0:p1
            for p3 = 0:p2
                for p4 = 0:p3
                    for p5 = 0:p4
                        EXP(:,end+1) = [(p1-p2) (p2-p3) (p3-p4) (p4-p5) p5]';
                    end
                end
            end
        end
    end

elseif N_var == 6
    for p1 = 0:N_order
        for p2 = 0:p1
            for p3 = 0:p2
                for p4 = 0:p3
                    for p5 = 0:p4
                        for p6 = 0:p5
                            EXP(:,end+1) = [(p1-p2) (p2-p3) (p3-p4) (p4-p5) (p5-p6) p6]';
                        end
                    end
                end
            end
        end
    end
    
elseif N_var == 7
    for p1 = 0:N_order
        for p2 = 0:p1
            for p3 = 0:p2
                for p4 = 0:p3
                    for p5 = 0:p4
                        for p6 = 0:p5
                            for p7 = 0:p6
                                EXP(:,end+1) = [(p1-p2) (p2-p3) (p3-p4) (p4-p5) (p5-p6) (p6-p7) p7]';
                            end
                        end
                    end
                end
            end
        end
    end
    
elseif N_var == 8
    for p1 = 0:N_order
        for p2 = 0:p1
            for p3 = 0:p2
                for p4 = 0:p3
                    for p5 = 0:p4
                        for p6 = 0:p5
                            for p7 = 0:p6
                                for p8 = 0:p7
                                    EXP(:,end+1) = [(p1-p2) (p2-p3) (p3-p4) (p4-p5) (p5-p6) (p6-p7) (p7-p8) p8]';
                                end
                            end
                        end
                    end
                end
            end
        end
    end
    
elseif N_var == 9
    for p1 = 0:N_order
        for p2 = 0:p1
            for p3 = 0:p2
                for p4 = 0:p3
                    for p5 = 0:p4
                        for p6 = 0:p5
                            for p7 = 0:p6
                                for p8 = 0:p7
                                    for p9 = 0:p8
                                        EXP(:,end+1) = [(p1-p2) (p2-p3) (p3-p4) (p4-p5) (p5-p6) (p6-p7) (p7-p8) (p8-p9) p9]';
                                    end
                                end
                            end
                        end
                    end
                end
            end
        end
    end
    
elseif N_var == 10
    for p1 = 0:N_order
        for p2 = 0:p1
            for p3 = 0:p2
                for p4 = 0:p3
                    for p5 = 0:p4
                        for p6 = 0:p5
                            for p7 = 0:p6
                                for p8 = 0:p7
                                    for p9 = 0:p8
                                        for p10 = 0:p9
                                            EXP(:,end+1) = [(p1-p2) (p2-p3) (p3-p4) (p4-p5) (p5-p6) (p6-p7) (p7-p8) (p8-p9) (p9-p10) p10]';
                                        end
                                    end
                                end
                            end
                        end
                    end
                end
            end
        end
    end
    
elseif N_var == 11
    for p1 = 0:N_order
        for p2 = 0:p1
            for p3 = 0:p2
                for p4 = 0:p3
                    for p5 = 0:p4
                        for p6 = 0:p5
                            for p7 = 0:p6
                                for p8 = 0:p7
                                    for p9 = 0:p8
                                        for p10 = 0:p9
                                            for p11 = 0:p10
                                                EXP(:,end+1) = [(p1-p2) (p2-p3) (p3-p4) (p4-p5) (p5-p6) (p6-p7) (p7-p8) (p8-p9) (p9-p10) (p10-p11) p11]';
                                            end
                                        end
                                    end
                                end
                            end
                        end
                    end
                end
            end
        end
    end
    
elseif N_var == 12
    for p1 = 0:N_order
        for p2 = 0:p1
            for p3 = 0:p2
                for p4 = 0:p3
                    for p5 = 0:p4
                        for p6 = 0:p5
                            for p7 = 0:p6
                                for p8 = 0:p7
                                    for p9 = 0:p8
                                        for p10 = 0:p9
                                            for p11 = 0:p10
                                                for p12 = 0:p11
                                                    EXP(:,end+1) = [(p1-p2) (p2-p3) (p3-p4) (p4-p5) (p5-p6) ...
                                                        (p6-p7) (p7-p8) (p8-p9) (p9-p10) (p10-p11) (p11-p12) p12]';
                                                end
                                            end
                                        end
                                    end
                                end
                            end
                        end
                    end
                end
            end
        end
    end
    
elseif N_var == 13
    for p1 = 0:N_order
        for p2 = 0:p1
            for p3 = 0:p2
                for p4 = 0:p3
                    for p5 = 0:p4
                        for p6 = 0:p5
                            for p7 = 0:p6
                                for p8 = 0:p7
                                    for p9 = 0:p8
                                        for p10 = 0:p9
                                            for p11 = 0:p10
                                                for p12 = 0:p11
                                                    for p13 = 0:p12
                                                        EXP(:,end+1) = [(p1-p2) (p2-p3) (p3-p4) (p4-p5) (p5-p6) ...
                                                        (p6-p7) (p7-p8) (p8-p9) (p9-p10) (p10-p11) (p11-p12) (p12-p13) p13]';
                                                    end
                                                end
                                            end
                                        end
                                    end
                                end
                            end
                        end
                    end
                end
            end
        end
    end
    
elseif N_var == 14
    for p1 = 0:N_order
        for p2 = 0:p1
            for p3 = 0:p2
                for p4 = 0:p3
                    for p5 = 0:p4
                        for p6 = 0:p5
                            for p7 = 0:p6
                                for p8 = 0:p7
                                    for p9 = 0:p8
                                        for p10 = 0:p9
                                            for p11 = 0:p10
                                                for p12 = 0:p11
                                                    for p13 = 0:p12
                                                        for p14 = 0:p13
                                                            EXP(:,end+1) = [(p1-p2) (p2-p3) (p3-p4) (p4-p5) (p5-p6) ...
                                                            (p6-p7) (p7-p8) (p8-p9) (p9-p10) (p10-p11) (p11-p12) (p12-p13) (p13-p14) p14]';
                                                        end
                                                    end
                                                end
                                            end
                                        end
                                    end
                                end
                            end
                        end
                    end
                end
            end
        end
    end
    
elseif N_var == 15
    for p1 = 0:N_order
        for p2 = 0:p1
            for p3 = 0:p2
                for p4 = 0:p3
                    for p5 = 0:p4
                        for p6 = 0:p5
                            for p7 = 0:p6
                                for p8 = 0:p7
                                    for p9 = 0:p8
                                        for p10 = 0:p9
                                            for p11 = 0:p10
                                                for p12 = 0:p11
                                                    for p13 = 0:p12
                                                        for p14 = 0:p13
                                                            for p15 = 0:p14
                                                                EXP(:,end+1) = [(p1-p2) (p2-p3) (p3-p4) (p4-p5) (p5-p6) ...
                                                                (p6-p7) (p7-p8) (p8-p9) (p9-p10) (p10-p11) (p11-p12) (p12-p13) (p13-p14) (p14-p15) p15]';
                                                            end
                                                        end
                                                    end
                                                end
                                            end
                                        end
                                    end
                                end
                            end
                        end
                    end
                end
            end
        end
    end
    
elseif N_var == 16
    for p1 = 0:N_order
        for p2 = 0:p1
            for p3 = 0:p2
                for p4 = 0:p3
                    for p5 = 0:p4
                        for p6 = 0:p5
                            for p7 = 0:p6
                                for p8 = 0:p7
                                    for p9 = 0:p8
                                        for p10 = 0:p9
                                            for p11 = 0:p10
                                                for p12 = 0:p11
                                                    for p13 = 0:p12
                                                        for p14 = 0:p13
                                                            for p15 = 0:p14
                                                                for p16 = 0:p15
                                                                    EXP(:,end+1) = [(p1-p2) (p2-p3) (p3-p4) (p4-p5) (p5-p6) ...
                                                                    (p6-p7) (p7-p8) (p8-p9) (p9-p10) (p10-p11) (p11-p12) ...
                                                                    (p12-p13) (p13-p14) (p14-p15) (p15-p16) p16]';
                                                                end
                                                            end
                                                        end
                                                    end
                                                end
                                            end
                                        end
                                    end
                                end
                            end
                        end
                    end
                end
            end
        end
    end
    
elseif N_var == 17
    for p1 = 0:N_order
        for p2 = 0:p1
            for p3 = 0:p2
                for p4 = 0:p3
                    for p5 = 0:p4
                        for p6 = 0:p5
                            for p7 = 0:p6
                                for p8 = 0:p7
                                    for p9 = 0:p8
                                        for p10 = 0:p9
                                            for p11 = 0:p10
                                                for p12 = 0:p11
                                                    for p13 = 0:p12
                                                        for p14 = 0:p13
                                                            for p15 = 0:p14
                                                                for p16 = 0:p15
                                                                    for p17 = 0:p16
                                                                        EXP(:,end+1) = [(p1-p2) (p2-p3) (p3-p4) (p4-p5) (p5-p6) ...
                                                                        (p6-p7) (p7-p8) (p8-p9) (p9-p10) (p10-p11) (p11-p12) ...
                                                                        (p12-p13) (p13-p14) (p14-p15) (p15-p16) (p16-p17) p17]';
                                                                    end
                                                                end
                                                            end
                                                        end
                                                    end
                                                end
                                            end
                                        end
                                    end
                                end
                            end
                        end
                    end
                end
            end
        end
    end
    
elseif N_var == 18
    for p1 = 0:N_order
        for p2 = 0:p1
            for p3 = 0:p2
                for p4 = 0:p3
                    for p5 = 0:p4
                        for p6 = 0:p5
                            for p7 = 0:p6
                                for p8 = 0:p7
                                    for p9 = 0:p8
                                        for p10 = 0:p9
                                            for p11 = 0:p10
                                                for p12 = 0:p11
                                                    for p13 = 0:p12
                                                        for p14 = 0:p13
                                                            for p15 = 0:p14
                                                                for p16 = 0:p15
                                                                    for p17 = 0:p16
                                                                        for p18 = 0:p17
                                                                            EXP(:,end+1) = [(p1-p2) (p2-p3) (p3-p4) (p4-p5) (p5-p6) ...
                                                                            (p6-p7) (p7-p8) (p8-p9) (p9-p10) (p10-p11) (p11-p12) ...
                                                                            (p12-p13) (p13-p14) (p14-p15) (p15-p16) (p16-p17) (p17-p18) p18]';
                                                                        end
                                                                    end
                                                                end
                                                            end
                                                        end
                                                    end
                                                end
                                            end
                                        end
                                    end
                                end
                            end
                        end
                    end
                end
            end
        end
    end
    
elseif N_var == 19
    for p1 = 0:N_order
        for p2 = 0:p1
            for p3 = 0:p2
                for p4 = 0:p3
                    for p5 = 0:p4
                        for p6 = 0:p5
                            for p7 = 0:p6
                                for p8 = 0:p7
                                    for p9 = 0:p8
                                        for p10 = 0:p9
                                            for p11 = 0:p10
                                                for p12 = 0:p11
                                                    for p13 = 0:p12
                                                        for p14 = 0:p13
                                                            for p15 = 0:p14
                                                                for p16 = 0:p15
                                                                    for p17 = 0:p16
                                                                        for p18 = 0:p17
                                                                            for p19 = 0:p18
                                                                                EXP(:,end+1) = [(p1-p2) (p2-p3) (p3-p4) (p4-p5) (p5-p6) ...
                                                                                (p6-p7) (p7-p8) (p8-p9) (p9-p10) (p10-p11) (p11-p12) ...
                                                                                (p12-p13) (p13-p14) (p14-p15) (p15-p16) (p16-p17) (p17-p18) (p18-p19) p19]';
                                                                            end
                                                                        end
                                                                    end
                                                                end
                                                            end
                                                        end
                                                    end
                                                end
                                            end
                                        end
                                    end
                                end
                            end
                        end
                    end
                end
            end
        end
    end
    
elseif N_var == 20
    for p1 = 0:N_order
        for p2 = 0:p1
            for p3 = 0:p2
                for p4 = 0:p3
                    for p5 = 0:p4
                        for p6 = 0:p5
                            for p7 = 0:p6
                                for p8 = 0:p7
                                    for p9 = 0:p8
                                        for p10 = 0:p9
                                            for p11 = 0:p10
                                                for p12 = 0:p11
                                                    for p13 = 0:p12
                                                        for p14 = 0:p13
                                                            for p15 = 0:p14
                                                                for p16 = 0:p15
                                                                    for p17 = 0:p16
                                                                        for p18 = 0:p17
                                                                            for p19 = 0:p18
                                                                                for p20 = 0:p19
                                                                                    EXP(:,end+1) = [(p1-p2) (p2-p3) (p3-p4) (p4-p5) (p5-p6) (p6-p7) ...
                                                                                        (p7-p8) (p8-p9) (p9-p10) (p10-p11) (p11-p12) (p12-p13) (p13-p14) ...
                                                                                        (p14-p15) (p15-p16) (p16-p17) (p17-p18) (p18-p19) (p19-p20) p20]';
                                                                                end
                                                                            end
                                                                        end
                                                                    end
                                                                end
                                                            end
                                                        end
                                                    end
                                                end
                                            end
                                        end
                                    end
                                end
                            end
                        end
                    end
                end
            end
        end
    end
    
elseif N_var == 21
    for p1 = 0:N_order
        for p2 = 0:p1
            for p3 = 0:p2
                for p4 = 0:p3
                    for p5 = 0:p4
                        for p6 = 0:p5
                            for p7 = 0:p6
                                for p8 = 0:p7
                                    for p9 = 0:p8
                                        for p10 = 0:p9
                                            for p11 = 0:p10
                                                for p12 = 0:p11
                                                    for p13 = 0:p12
                                                        for p14 = 0:p13
                                                            for p15 = 0:p14
                                                                for p16 = 0:p15
                                                                    for p17 = 0:p16
                                                                        for p18 = 0:p17
                                                                            for p19 = 0:p18
                                                                                for p20 = 0:p19
                                                                                    for p21 = 0:p20
                                                                                        EXP(:,end+1) = [(p1-p2) (p2-p3) (p3-p4) (p4-p5) (p5-p6) (p6-p7) ...
                                                                                            (p7-p8) (p8-p9) (p9-p10) (p10-p11) (p11-p12) (p12-p13) (p13-p14) ...
                                                                                            (p14-p15) (p15-p16) (p16-p17) (p17-p18) (p18-p19) (p19-p20) ...
                                                                                            (p20-p21) p21]';
                                                                                    end
                                                                                end
                                                                            end
                                                                        end
                                                                    end
                                                                end
                                                            end
                                                        end
                                                    end
                                                end
                                            end
                                        end
                                    end
                                end
                            end
                        end
                    end
                end
            end
        end
    end
    
elseif N_var == 22
    for p1 = 0:N_order
        for p2 = 0:p1
            for p3 = 0:p2
                for p4 = 0:p3
                    for p5 = 0:p4
                        for p6 = 0:p5
                            for p7 = 0:p6
                                for p8 = 0:p7
                                    for p9 = 0:p8
                                        for p10 = 0:p9
                                            for p11 = 0:p10
                                                for p12 = 0:p11
                                                    for p13 = 0:p12
                                                        for p14 = 0:p13
                                                            for p15 = 0:p14
                                                                for p16 = 0:p15
                                                                    for p17 = 0:p16
                                                                        for p18 = 0:p17
                                                                            for p19 = 0:p18
                                                                                for p20 = 0:p19
                                                                                    for p21 = 0:p20
                                                                                        for p22 = 0:p21
                                                                                            EXP(:,end+1) = [(p1-p2) (p2-p3) (p3-p4) (p4-p5) (p5-p6) (p6-p7) ...
                                                                                                (p7-p8) (p8-p9) (p9-p10) (p10-p11) (p11-p12) (p12-p13) (p13-p14) ...
                                                                                                (p14-p15) (p15-p16) (p16-p17) (p17-p18) (p18-p19) (p19-p20) ...
                                                                                                (p20-p21) (p21-p22) p22]';
                                                                                        end
                                                                                    end
                                                                                end
                                                                            end
                                                                        end
                                                                    end
                                                                end
                                                            end
                                                        end
                                                    end
                                                end
                                            end
                                        end
                                    end
                                end
                            end
                        end
                    end
                end
            end
        end
    end
    
elseif N_var == 23
    for p1 = 0:N_order
        for p2 = 0:p1
            for p3 = 0:p2
                for p4 = 0:p3
                    for p5 = 0:p4
                        for p6 = 0:p5
                            for p7 = 0:p6
                                for p8 = 0:p7
                                    for p9 = 0:p8
                                        for p10 = 0:p9
                                            for p11 = 0:p10
                                                for p12 = 0:p11
                                                    for p13 = 0:p12
                                                        for p14 = 0:p13
                                                            for p15 = 0:p14
                                                                for p16 = 0:p15
                                                                    for p17 = 0:p16
                                                                        for p18 = 0:p17
                                                                            for p19 = 0:p18
                                                                                for p20 = 0:p19
                                                                                    for p21 = 0:p20
                                                                                        for p22 = 0:p21
                                                                                            for p23 = 0:p22
                                                                                                EXP(:,end+1) = [(p1-p2) (p2-p3) (p3-p4) (p4-p5) (p5-p6) (p6-p7) ...
                                                                                                    (p7-p8) (p8-p9) (p9-p10) (p10-p11) (p11-p12) (p12-p13) (p13-p14) ...
                                                                                                    (p14-p15) (p15-p16) (p16-p17) (p17-p18) (p18-p19) (p19-p20) ...
                                                                                                    (p20-p21) (p21-p22) (p22-p23) p23]';
                                                                                            end
                                                                                        end
                                                                                    end
                                                                                end
                                                                            end
                                                                        end
                                                                    end
                                                                end
                                                            end
                                                        end
                                                    end
                                                end
                                            end
                                        end
                                    end
                                end
                            end
                        end
                    end
                end
            end
        end
    end
    
elseif N_var == 24
    for p1 = 0:N_order
        for p2 = 0:p1
            for p3 = 0:p2
                for p4 = 0:p3
                    for p5 = 0:p4
                        for p6 = 0:p5
                            for p7 = 0:p6
                                for p8 = 0:p7
                                    for p9 = 0:p8
                                        for p10 = 0:p9
                                            for p11 = 0:p10
                                                for p12 = 0:p11
                                                    for p13 = 0:p12
                                                        for p14 = 0:p13
                                                            for p15 = 0:p14
                                                                for p16 = 0:p15
                                                                    for p17 = 0:p16
                                                                        for p18 = 0:p17
                                                                            for p19 = 0:p18
                                                                                for p20 = 0:p19
                                                                                    for p21 = 0:p20
                                                                                        for p22 = 0:p21
                                                                                            for p23 = 0:p22
                                                                                                for p24 = 0:p23
                                                                                                    EXP(:,end+1) = [(p1-p2) (p2-p3) (p3-p4) (p4-p5) (p5-p6) (p6-p7) ...
                                                                                                        (p7-p8) (p8-p9) (p9-p10) (p10-p11) (p11-p12) (p12-p13) (p13-p14) ...
                                                                                                        (p14-p15) (p15-p16) (p16-p17) (p17-p18) (p18-p19) (p19-p20) ...
                                                                                                        (p20-p21) (p21-p22) (p22-p23) (p23-p24) p24]';
                                                                                                end
                                                                                            end
                                                                                        end
                                                                                    end
                                                                                end
                                                                            end
                                                                        end
                                                                    end
                                                                end
                                                            end
                                                        end
                                                    end
                                                end
                                            end
                                        end
                                    end
                                end
                            end
                        end
                    end
                end
            end
        end
    end
    
elseif N_var == 25
    for p1 = 0:N_order
        for p2 = 0:p1
            for p3 = 0:p2
                for p4 = 0:p3
                    for p5 = 0:p4
                        for p6 = 0:p5
                            for p7 = 0:p6
                                for p8 = 0:p7
                                    for p9 = 0:p8
                                        for p10 = 0:p9
                                            for p11 = 0:p10
                                                for p12 = 0:p11
                                                    for p13 = 0:p12
                                                        for p14 = 0:p13
                                                            for p15 = 0:p14
                                                                for p16 = 0:p15
                                                                    for p17 = 0:p16
                                                                        for p18 = 0:p17
                                                                            for p19 = 0:p18
                                                                                for p20 = 0:p19
                                                                                    for p21 = 0:p20
                                                                                        for p22 = 0:p21
                                                                                            for p23 = 0:p22
                                                                                                for p24 = 0:p23
                                                                                                    for p25 = 0:p24
                                                                                                        EXP(:,end+1) = [(p1-p2) (p2-p3) (p3-p4) (p4-p5) (p5-p6) (p6-p7) ...
                                                                                                            (p7-p8) (p8-p9) (p9-p10) (p10-p11) (p11-p12) (p12-p13) (p13-p14) ...
                                                                                                            (p14-p15) (p15-p16) (p16-p17) (p17-p18) (p18-p19) (p19-p20) ...
                                                                                                            (p20-p21) (p21-p22) (p22-p23) (p23-p24) (p24-p25) p25]';
                                                                                                    end
                                                                                                end
                                                                                            end
                                                                                        end
                                                                                    end
                                                                                end
                                                                            end
                                                                        end
                                                                    end
                                                                end
                                                            end
                                                        end
                                                    end
                                                end
                                            end
                                        end
                                    end
                                end
                            end
                        end
                    end
                end
            end
        end
    end
    
end