function [Coeff_prod,bound_extra] = polyprod(Coeff1,Coeff2)

% [Coeff_prod,bound_extra] = polyprod(Coeff1,Coeff2,XRANGE) computes in the
% variable Coeff_prod the coefficient of the product of two polynomials 
% defined by Coeff1 and Coeff2 up to a maximum order fixed by the Taylor 
% model representation. The truncation error is taken into account by the
% bounded variable bound_extra

% Last revised: 31.10.2016. 
% Created by: Riccardo Trinchero, Paolo Manfredi, Igor S. Stievano.
%
% TMsim - Copyright (c) 2016 Riccardo Trinchero, Paolo Manfredi, Igor
% Stievano All rights reserved.
%  
% Embedding the program code in any commercial software is strictly
% prohibited. If the code is used in a scientific work, then reference
% should me made to the following publication.
%  
% [1] R. Trinchero, P. Manfredi, T. Ding, I. S. Stievano, "Combined
% Parametric and Worst Case Circuit Analysis via Taylor Models", IEEE
% Trans. Circuits and Systems - I: regular papers, Vol. 63, No. 7, pp.
% 1067-1078, Jul. 2016. PDFlink
%  
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
% IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
% THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
% PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


global EXP 
global EXP_extra

dim_std = size(EXP,2);
dim_coef = size(EXP_extra,2);
Coeff_extra= zeros(1,dim_coef);

% exp index matrix
global EXPINDEX;

if isempty(EXPINDEX)
    EXPINDEX = reshape(EXP_extra,1,[]);
end


index1 = find(Coeff1~=0);
index2 = find(Coeff2~=0);

N1 = length(index1);
N2 = length(index2);

for ii = 1:N1
    for jj = 1:N2
        
        tmp = EXP(:,index1(ii)) + EXP(:,index2(jj));

        ind = strfind(EXPINDEX,tmp.');

        index_prod = ind((ind-1)/size(EXP_extra,1) == fix((ind-1)/size(EXP_extra,1)));
        if index_prod>1
            index_prod = (index_prod-1)/size(EXP_extra,1)+1;
        end
        
        Coeff_extra(index_prod) = Coeff_extra(index_prod)+Coeff1(index1(ii)) * Coeff2(index2(jj));
    end
end

Coeff_prod = Coeff_extra(1:dim_std);
Coeff_extra(1:dim_std) = 0;

if ~isempty(find(Coeff_extra))
    
    % BN bound
    bound_extra = TM_bound_BN_extra(Coeff_extra,EXP_extra);
    
else
    bound_extra = [0,0];
end
