function [] = initialize_TMsim(Norder,Nvar)

% [] = initialize_TMsim(Norder,Nvar) inizializes the TMsim toolbox for the
% Taylor Model variables and calculations.
% Norder: order of the polynomial expansion
% Nvar: number of independent variables in the polynomial expansion

% Last revised: 31.10.2016. 
% Created by: Riccardo Trinchero, Paolo Manfredi, Igor S. Stievano.
%
% TMsim - Copyright (c) 2016 Riccardo Trinchero, Paolo Manfredi, Igor
% Stievano All rights reserved.
%  
% Embedding the program code in any commercial software is strictly
% prohibited. If the code is used in a scientific work, then reference
% should me made to the following publication.
%  
% [1] R. Trinchero, P. Manfredi, T. Ding, I. S. Stievano, "Combined
% Parametric and Worst Case Circuit Analysis via Taylor Models", IEEE
% Trans. Circuits and Systems - I: regular papers, Vol. 63, No. 7, pp.
% 1067-1078, Jul. 2016. PDFlink
%  
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
% IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
% THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
% PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


% Expansion order
global N_order; N_order = Norder; 

% polinomial degree matrix
global EXP; EXP = matrix_EXP(Nvar,N_order); 

% polinomial degree matrix for the extra terms
global EXP_extra; EXP_extra = matrix_EXP(Nvar,2*N_order);

% parameter variations between [-1,1]
global XRANGE; XRANGE = [-1*ones(Nvar,1),ones(Nvar,1)]; 

%Bernstein Poly
global BN_Base; BN_Base = BN_Basis(N_order,-1,1);
global BN_Base_extra; BN_Base_extra = BN_Basis(2*N_order,-1,1);

display('TMsim Initialized...');



